Blob Blame History Raw
diff -up shadow-4.1.0/src/groupadd.c.redhat shadow-4.1.0/src/groupadd.c
--- shadow-4.1.0/src/groupadd.c.redhat	2007-11-24 23:41:19.000000000 +0100
+++ shadow-4.1.0/src/groupadd.c	2007-12-12 12:15:00.000000000 +0100
@@ -74,6 +74,7 @@ static char *Prog;
 static int oflg = 0;		/* permit non-unique group ID to be specified with -g */
 static int gflg = 0;		/* ID value for the new group */
 static int fflg = 0;		/* if group already exists, do nothing and exit(0) */
+static int rflg = 0;            /* for adding system accounts (Red Hat) */
 
 /* local function prototypes */
 static void usage (void);
@@ -100,6 +101,7 @@ static void usage (void)
 			   "Options:\n"
 			   "  -f, --force                   force exit with success status if the\n"
 			   "                                specified group already exists\n"
+                           "  -r,                           create system account\n"
 			   "  -g, --gid GID                 use GID for the new group\n"
 			   "  -h, --help                    display this help message and exit\n"
 			   "  -K, --key KEY=VALUE           overrides /etc/login.defs defaults\n"
@@ -198,8 +200,13 @@ static void find_new_gid (void)
 	const struct group *grp;
 	gid_t gid_min, gid_max;
 
-	gid_min = getdef_unum ("GID_MIN", 1000);
-	gid_max = getdef_unum ("GID_MAX", 60000);
+	if (!rflg) {
+		gid_min = getdef_unum ("GID_MIN", 500);
+                gid_max = getdef_unum ("GID_MAX", 60000);
+        } else {
+                gid_min = 1;
+                gid_max = getdef_unum ("GID_MIN", 500) - 1;
+        }
 
 	/*
 	 * Start with some GID value if the user didn't provide us with
@@ -430,7 +437,7 @@ int main (int argc, char **argv)
 		};
 
 		while ((c =
-			getopt_long (argc, argv, "fg:hK:o", long_options,
+			getopt_long (argc, argv, "frg:hK:o", long_options,
 				     &option_index)) != -1) {
 			switch (c) {
 			case 'f':
@@ -443,6 +450,12 @@ int main (int argc, char **argv)
 				 */
 				fflg++;
 				break;
+	                case 'r':
+	                        /*
+	                         * create a system group
+	                         */
+	                        rflg++;
+	                        break;
 			case 'g':
 				gflg++;
 				group_id = get_gid (optarg);
diff -up shadow-4.1.0/man/chpasswd.8.redhat shadow-4.1.0/man/chpasswd.8
diff -up shadow-4.1.0/man/newusers.8.redhat shadow-4.1.0/man/newusers.8
diff -up shadow-4.1.0/man/useradd.8.redhat shadow-4.1.0/man/useradd.8
--- shadow-4.1.0/man/useradd.8.redhat	2007-12-10 00:07:10.000000000 +0100
+++ shadow-4.1.0/man/useradd.8	2007-12-12 12:05:54.000000000 +0100
@@ -25,9 +25,9 @@ When invoked without the
 \fB\-D\fR
 option, the
 \fBuseradd\fR
-command creates a new user account using the values specified on the command line plus the default values from the system\. Depending on command line options, the
+command creates a new user account using the values specified on the command line and the default values from the system. Depending on command line options, the 
 \fBuseradd\fR
-command will update system files and may also create the new user\'s home directory and copy initial files\.
+command will update system files and may also create the new user's home directory and copy initial files. The version provided with Red Hat Linux will create a group for each user added to the system by default.
 .SH "OPTIONS"
 .PP
 The options which apply to the
@@ -84,7 +84,7 @@ The number of days after a password expi
 .PP
 \fB\-g\fR, \fB\-\-gid\fR \fIGROUP\fR
 .RS 4
-The group name or number of the user\'s initial login group\. The group name must exist\. A group number must refer to an already existing group\. The default group number is 1 or whatever is specified in
+The group name or number of the user\'s initial login group\. The group name must exist\. A group number must refer to an already existing group\.
 \fI/etc/default/useradd\fR\.
 .RE
 .PP
@@ -100,6 +100,13 @@ option\. The default is for the user to 
 Display help message and exit\.
 .RE
 .PP
+\fB-M\fR
+.RS 4
+The user\'s home directory will not be created, even if the system wide settings from 
+\fI/etc/login.defs\fR 
+is to create home dirs\.
+.RE
+.PP
 \fB\-m\fR, \fB\-\-create\-home\fR
 .RS 4
 The user\'s home directory will be created if it does not exist\. The files contained in
@@ -119,6 +126,13 @@ option is only valid in conjunction with
 option\. The default is to not create the directory and to not copy any files\.
 .RE
 .PP
+\fB-n\fR
+.RS 4
+A group having the same name as the user being added to the system will be created by default\. This option will turn off this Red Hat Linux specific behavior\. When this option is used, users by default will be placed in whatever group is specified in 
+\fI/etc/default/useradd\fR\. 
+If no default group is defined, group 1 will be used.
+.RE
+.PP
 \fB\-K\fR, \fB\-\-key\fR \fIKEY\fR=\fIVALUE\fR
 .RS 4
 Overrides /etc/login\.defs defaults (UID_MIN, UID_MAX, UMASK, PASS_MAX_DAYS and others)\.
@@ -150,6 +164,19 @@ The encrypted password, as returned by
 \fBcrypt\fR(3)\. The default is to disable the account\.
 .RE
 .PP
+\fB-r\fR
+.RS 4
+This flag is used to create a system account\. That is, a user with a UID lower than the value of UID_MIN defined in 
+\fI/etc/login.defs\fR 
+and whose password does not expire\. Note that 
+\fBuseradd\fR 
+will not create a home directory for such an user, regardless of the default setting in 
+\fI/etc/login.defs\fR\. 
+You have to specify 
+\fB-m\fR 
+option if you want a home directory for a system account to be created\. This is an option added by Red Hat\.
+.RE
+.PP
 \fB\-s\fR, \fB\-\-shell\fR \fISHELL\fR
 .RS 4
 The name of the user\'s login shell\. The default is to leave this field blank, which causes the system to select the default login shell\.
@@ -206,6 +233,8 @@ The name of a new user\'s login shell\.
 The system administrator is responsible for placing the default user files in the
 \fI/etc/skel/\fR
 directory\.
+.br
+This version of useradd was modified by Red Hat to suit Red Hat user/group conventions\.
 .SH "CAVEATS"
 .PP
 You may not add a user to a NIS or LDAP group\. This must be performed on the corresponding server\.
@@ -315,6 +344,11 @@ Secure user account information\.
 Group account information\.
 .RE
 .PP
+\fI/etc/gshadow\fR
+.RS 4
+Secure group account information\.
+.RE
+.PP
 \fI/etc/default/useradd\fR
 .RS 4
 Default values for account creation\.
diff -up shadow-4.1.0/man/groupadd.8.redhat shadow-4.1.0/man/groupadd.8
--- shadow-4.1.0/man/groupadd.8.redhat	2007-12-10 00:06:00.000000000 +0100
+++ shadow-4.1.0/man/groupadd.8	2007-12-12 12:11:23.000000000 +0100
@@ -14,7 +14,7 @@
 groupadd - create a new group
 .SH "SYNOPSIS"
 .HP 9
-\fBgroupadd\fR [\-g\ \fIGID\fR\ [\-o]] [\-f] [\-K\ \fIKEY\fR=\fIVALUE\fR] \fIgroup\fR
+\fBgroupadd\fR [\-g\ \fIgid\fR\ [\-o]] [\-r] [\-f] [\-K\ \fIKEY\fR=\fIVALUE\fR] \fIgroup\fR
 .SH "DESCRIPTION"
 .PP
 The
@@ -34,11 +34,22 @@ This option causes the command to simply
 is turned off)\.
 .RE
 .PP
+\fB-r\fR
+.RS 4
+This flag instructs 
+\fBgroupadd\fR 
+to add a system account\.  The first available 
+\fIgid\fR 
+lower than 499 will be automatically selected unless the 
+\fB-g\fR 
+option is also given on the command line\. This is an option added by Red Hat\.
+.RE
+.PP
 \fB\-g\fR \fIGID\fR
 .RS 4
 The numerical value of the group\'s ID\. This value must be unique, unless the
 \fB\-o\fR
-option is used\. The value must be non\-negative\. The default is to use the smallest ID value greater than 999 and greater than every other group\. Values between 0 and 999 are typically reserved for system accounts\.
+option is used\. The value must be non\-negative\. The default is to use the smallest ID value greater than 500 and greater than every other group\. Values between 0 and 499 are typically reserved for system accounts\.
 .RE
 .PP
 \fB\-h\fR, \fB\-\-help\fR
diff -up shadow-4.1.0/src/useradd.c.redhat shadow-4.1.0/src/useradd.c
--- shadow-4.1.0/src/useradd.c.redhat	2007-12-09 23:43:09.000000000 +0100
+++ shadow-4.1.0/src/useradd.c	2007-12-12 12:19:34.000000000 +0100
@@ -81,7 +81,7 @@
 static gid_t def_group = 100;
 static const char *def_gname = "other";
 static const char *def_home = "/home";
-static const char *def_shell = "";
+static const char *def_shell = "/sbin/nologin";
 static const char *def_template = SKEL_DIR;
 static const char *def_create_mail_spool = "no";
 
@@ -93,7 +93,7 @@ static char def_file[] = USER_DEFAULTS_F
 #define	VALID(s)	(strcspn (s, ":\n") == strlen (s))
 
 static const char *user_name = "";
-static const char *user_pass = "!";
+static const char *user_pass = "!!";
 static uid_t user_id;
 static gid_t user_gid;
 static const char *user_comment = "";
@@ -124,8 +124,10 @@ static int
     Gflg = 0,			/* secondary group set for new account */
     kflg = 0,			/* specify a directory to fill new user directory */
     mflg = 0,			/* create user's home directory if it doesn't exist */
-    nflg = 0,			/* create a group having the same name as the user */
+    Mflg = 0,                   /* do NOT create user's home directory no matter what */
+    nflg = 0,                   /* do NOT create a group having the same name as the user */
     oflg = 0,			/* permit non-unique user ID to be specified with -u */
+    rflg = 0,                   /* create a system account */
     sflg = 0,			/* shell program for new account */
     uflg = 0;			/* specify user ID for new account */
 
@@ -632,6 +634,8 @@ static void usage (void)
 			   "  -K, --key KEY=VALUE           overrides /etc/login.defs defaults\n"
 			   "  -m, --create-home             create home directory for the new user\n"
 			   "                                account\n"
+                           "  -M,                       do not create user's home directory(overrides /etc/login.defs)\n"
+                           "  -r,                       create system account\n"
 			   "  -o, --non-unique              allow create user with duplicate\n"
 			   "                                (non-unique) UID\n"
 			   "  -p, --password PASSWORD       use encrypted password for the new user\n"
@@ -684,11 +688,20 @@ static void new_spent (struct spwd *spen
 	spent->sp_namp = (char *) user_name;
 	spent->sp_pwdp = (char *) user_pass;
 	spent->sp_lstchg = time ((time_t *) 0) / SCALE;
-	spent->sp_min = scale_age (getdef_num ("PASS_MIN_DAYS", -1));
-	spent->sp_max = scale_age (getdef_num ("PASS_MAX_DAYS", -1));
-	spent->sp_warn = scale_age (getdef_num ("PASS_WARN_AGE", -1));
-	spent->sp_inact = scale_age (def_inactive);
-	spent->sp_expire = scale_age (user_expire);
+	if (!rflg) {
+		spent->sp_min = scale_age (getdef_num ("PASS_MIN_DAYS", -1));
+		spent->sp_max = scale_age (getdef_num ("PASS_MAX_DAYS", -1));
+		spent->sp_warn = scale_age (getdef_num ("PASS_WARN_AGE", -1));
+		spent->sp_inact = scale_age (def_inactive);
+		spent->sp_expire = scale_age (user_expire);
+	}
+	else {
+		spent->sp_min = scale_age(-1);
+                spent->sp_max = scale_age(-1);
+                spent->sp_warn = scale_age(-1);
+                spent->sp_inact = scale_age(-1);
+        	spent->sp_expire = scale_age(-1);
+	}
 	spent->sp_flag = -1;
 }
 
@@ -821,8 +834,14 @@ static void find_new_uid (void)
 	const struct passwd *pwd;
 	uid_t uid_min, uid_max;
 
-	uid_min = getdef_unum ("UID_MIN", 1000);
-	uid_max = getdef_unum ("UID_MAX", 60000);
+        if (!rflg) {
+                uid_min = getdef_unum ("UID_MIN", 500);
+                uid_max = getdef_unum ("UID_MAX", 60000);
+        }
+        else {
+                uid_min = 1;
+                uid_max = getdef_unum ("UID_MIN", 500) - 1;
+        }
 
 	/*
 	 * Start with some UID value if the user didn't provide us with
@@ -852,7 +871,7 @@ static void find_new_uid (void)
 #endif
 			exit (E_NAME_IN_USE);
 		}
-		if (uflg && user_id == pwd->pw_uid) {
+		if (!oflg && uflg && user_id == pwd->pw_uid) {
 			fprintf (stderr, _("%s: UID %u is not unique\n"),
 				 Prog, (unsigned int) user_id);
 #ifdef WITH_AUDIT
@@ -908,8 +927,13 @@ static void find_new_gid ()
 	const struct group *grp;
 	gid_t gid_min, gid_max;
 
-	gid_min = getdef_num ("GID_MIN", 500);
-	gid_max = getdef_num ("GID_MAX", 60000);
+        if (!rflg) {
+                gid_min = getdef_unum ("GID_MIN", 500);
+                gid_max = getdef_unum ("GID_MAX", 60000);
+        } else {
+                gid_min = 1;
+                gid_max = getdef_unum ("GID_MIN", 500) - 1;
+        }
 
 	/*
 	 * Start with some GID value if the user didn't provide us with
@@ -1008,7 +1032,7 @@ static void process_flags (int argc, cha
 			{NULL, 0, NULL, '\0'}
 		};
 		while ((c =
-			getopt_long (argc, argv, "b:c:d:De:f:g:G:k:K:mMop:s:u:",
+			getopt_long (argc, argv, "b:c:d:De:f:g:G:k:K:mMnrop:s:u:",
 				     long_options, NULL)) != -1) {
 			switch (c) {
 			case 'b':
@@ -1177,6 +1201,15 @@ static void process_flags (int argc, cha
 				user_id = get_uid (optarg);
 				uflg++;
 				break;
+                        case 'n':
+                                nflg++;
+                                break;
+                        case 'r':
+                                rflg++;
+                                break;
+                        case 'M':
+                                Mflg++;
+                                break;
 			default:
 				usage ();
 			}
@@ -1184,6 +1217,9 @@ static void process_flags (int argc, cha
 		}
 	}
 
+        if (mflg && Mflg) /* the admin is not decided .. create or not ? */
+               usage();
+
 	/*
 	 * Certain options are only valid in combination with others.
 	 * Check it here so that they can be specified in any order.
@@ -1698,6 +1734,14 @@ int main (int argc, char **argv)
 	}
 #endif				/* USE_PAM */
 
+        if (!rflg) /* for system accounts defaults are ignored and we
+                    * do not create a home dir -- gafton */
+        	if (getdef_bool("CREATE_HOME"))
+               		mflg = 1;
+
+        if (Mflg) /* absolutely sure that we do not create home dirs */
+        	mflg = 0;
+
 	/*
 	 * See if we are messing with the defaults file, or creating
 	 * a new user.
@@ -1728,7 +1772,7 @@ int main (int argc, char **argv)
 	 * to that group, use useradd -g username username.
 	 * --bero
 	 */
-	if (!gflg) {
+	if ( !(nflg || gflg) ) {
 		if (getgrnam (user_name)) { /* local, no need for xgetgrnam */
 			fprintf (stderr,
 				 _
@@ -1779,27 +1823,22 @@ int main (int argc, char **argv)
 				 ("%s: warning: the home directory already exists.\n"
 				  "Not copying any file from skel directory into it.\n"),
 				 Prog);
-
-	} else if (getdef_str ("CREATE_HOME")) {
-		/*
-		 * RedHat added the CREATE_HOME option in login.defs in their
-		 * version of shadow-utils (which makes -m the default, with
-		 * new -M option to turn it off). Unfortunately, this
-		 * changes the way useradd works (it can be run by scripts
-		 * expecting some standard behaviour), compared to other
-		 * Unices and other Linux distributions, and also adds a lot
-		 * of confusion :-(.
-		 * So we now recognize CREATE_HOME and give a warning here
-		 * (better than "configuration error ... notify administrator"
-		 * errors in every program that reads /etc/login.defs). -MM
-		 */
-		fprintf (stderr,
-			 _
-			 ("%s: warning: CREATE_HOME not supported, please use -m instead.\n"),
-			 Prog);
 	}
-
-	create_mail ();
+        /* Warning removed to protect the innocent. */
+        /*
+         * The whole idea about breaking some stupid scripts by creating a new
+         * variable is crap - I could care less about the scripts. Historically
+         * adduser type programs have always created the home directories and
+         * I don't like the idea of providing a script when we can fix the
+         * binary itself. And if the scripts are using the right options to the
+         * useradd then they will not break. If not, they depend on unspecified
+         * behavior and they will break, but they were broken anyway to begin
+         * with --gafton
+         */
+
+	/* Do not create mail directory for system accounts */
+	if( !rflg )
+		create_mail ();
 
 	close_files ();
 